'' =================================================================================================
''
''   File....... jm_hrtimer.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 30 APR 2013
''
'' =================================================================================================


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


var

  long cog

  long  timertix                                                ' result of timing
  long  tpin                                                    ' control pin
  
    
pub setup(p)

'' Start high-res timer using pin p for control

  kill

  tpin := p                                                     ' set control pin
  outa[tpin] := 0                                               ' disable
  dira[tpin] := 1
    
  cog := cognew(@hrtimer, @timertix) + 1                        ' start the cog

  return cog


pub kill

'' Stops timer cog (if running)

  if (cog)
    cogstop(cog - 1)
    cog := 0
    dira[tpin] := 0                                             ' free control pin

  timertix := 0


pub start

'' Enables pasm timer

  timertix := 0                                                 ' clear old result
  outa[tpin] := 1                                               ' enable/start timer


pub stop

'' Disables pasm timer
'' -- returns timing in system ticks

  outa[tpin] := 0                                               ' disable/stop timer
  repeat until (timertix)

  return timertix


pub run_time

'' Returns last timing result

  return timertix


dat

                        org     0

hrtimer                 mov     t1, par                         ' start of parameters
                        add     t1, #4                          ' skip over timing result
                        rdlong  t2, t1                          ' read sync pin
                        mov     syncmask, #1                    ' convert to mask
                        shl     syncmask, t2

                        mov     ctra, POS_DETECT                ' ctra measures high state
                        or      ctra, t2                        '  of sync pin
                        mov     frqa, #1

hrmain                  waitpne syncmask, syncmask              ' wait for low on sync    
                        mov     phsa, #0                        ' reset timer   

                        waitpeq syncmask, syncmask              ' wait for high on sync
                        nop
                        waitpne syncmask, syncmask              ' wait for low on sync

                        mov     t1, phsa                        ' copy timing
                        wrlong  t1, par                         ' report to hub

                        jmp     #hrmain

' --------------------------------------------------------------------------------------------------

POS_DETECT              long    %01000 << 26                    ' increment phsx on high

syncmask                res     1                               ' mask for control pin

t1                      res     1                               ' work vars
t2                      res     1

                        fit     496                                    

                        
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  