'' =================================================================================================
''
''   File....... jm_hrtimer_demo.spin 
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 30 APR 2013
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con 

  #(-1), IS_ON, IS_OFF

  #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term   : "fullduplexserial64"
  timer1 : "jm_hrtimer"
  timer2 : "jm_hrtimer"
  timer3 : "jm_hrtimer"   
 

var


pub main | t0

  term.start(RX1, TX1, %0000, 115_200)
  timer1.setup(16)  
  timer2.setup(17)
  timer3.setup(18) 

  ' press key on PST terminal to start demo

  term.rxflush
  term.rx

  term.tx(CLS)
  term.str(string("Timer Demo", CR, CR))

  ' enable all three timers at once via pins
  ' -- instead of .start method

  outa[18..16] := %111
  t0 := cnt
  
  waitcnt(t0 += clkfreq)                                        ' wait ~1s
  outa[16] := 0                                                 ' stop timer1
  term.dec(timer1.run_time)                                     ' report timing
  term.tx(CR)

  waitcnt(t0 += clkfreq)
  outa[17] := 0             
  term.dec(timer2.run_time)
  term.tx(CR)  

  waitcnt(t0 += clkfreq)
  outa[18] := 0             
  term.dec(timer3.run_time)
  term.tx(CR)

  repeat
    waitcnt(0)
  


con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}